"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BACKUP_DIR_WINSTORE = exports.BACKUP_DIR = void 0;
exports.initBackupAdapter = initBackupAdapter;
var electron_1 = require("electron");
var fs_1 = require("fs");
var ipc_events_const_1 = require("./shared-with-frontend/ipc-events.const");
var path = require("path");
var main_1 = require("electron-log/main");
exports.BACKUP_DIR = path.join(electron_1.app.getPath('userData'), "backups");
exports.BACKUP_DIR_WINSTORE = exports.BACKUP_DIR.replace('Roaming', "Local\\Packages\\53707johannesjo.SuperProductivity_ch45amy23cdv6\\LocalCache\\Roaming");
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function initBackupAdapter() {
    console.log('Saving backups to', exports.BACKUP_DIR);
    (0, main_1.log)('Saving backups to', exports.BACKUP_DIR);
    // BACKUP
    electron_1.ipcMain.on(ipc_events_const_1.IPC.BACKUP, backupData);
    // IS_BACKUP_AVAILABLE
    electron_1.ipcMain.handle(ipc_events_const_1.IPC.BACKUP_IS_AVAILABLE, function () {
        if (!(0, fs_1.existsSync)(exports.BACKUP_DIR)) {
            return false;
        }
        var files = (0, fs_1.readdirSync)(exports.BACKUP_DIR);
        if (!files.length) {
            return false;
        }
        var filesWithMeta = files.map(function (fileName) { return ({
            name: fileName,
            path: path.join(exports.BACKUP_DIR, fileName),
            folder: exports.BACKUP_DIR,
            created: (0, fs_1.statSync)(path.join(exports.BACKUP_DIR, fileName)).mtime.getTime(),
        }); });
        filesWithMeta.sort(function (a, b) { return a.created - b.created; });
        (0, main_1.log)('Avilable Backup Files: ', (filesWithMeta === null || filesWithMeta === void 0 ? void 0 : filesWithMeta.map) && filesWithMeta.map(function (f) { return f.path; }));
        return filesWithMeta.reverse()[0];
    });
    // RESTORE_BACKUP
    electron_1.ipcMain.handle(ipc_events_const_1.IPC.BACKUP_LOAD_DATA, function (ev, backupPath) {
        (0, main_1.log)('Reading backup file: ', backupPath);
        return (0, fs_1.readFileSync)(backupPath, { encoding: 'utf8' });
    });
}
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function backupData(ev, data) {
    if (!(0, fs_1.existsSync)(exports.BACKUP_DIR)) {
        (0, fs_1.mkdirSync)(exports.BACKUP_DIR);
    }
    var filePath = "".concat(exports.BACKUP_DIR, "/").concat(getDateStr(), ".json");
    try {
        var backup = JSON.stringify(data);
        (0, fs_1.writeFileSync)(filePath, backup);
    }
    catch (e) {
        (0, main_1.log)('Error while backing up');
        (0, main_1.error)(e);
    }
}
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function getDateStr() {
    var today = new Date();
    var dd = today.getDate();
    var mm = today.getMonth() + 1; // January is 0!
    var yyyy = today.getFullYear();
    var dds = dd < 10 ? '0' + dd : dd.toString();
    var mms = mm < 10 ? '0' + mm : mm.toString();
    return "".concat(yyyy, "-").concat(mms, "-").concat(dds);
}
//# sourceMappingURL=backup.js.map