"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.errorHandlerWithFrontendInform = void 0;
var main_window_1 = require("./main-window");
var ipc_events_const_1 = require("./shared-with-frontend/ipc-events.const");
var main_1 = require("electron-log/main");
var WAIT_FOR_WIN_TIMEOUT_DURATION = 4000;
var errorHandlerWithFrontendInform = function (e, additionalLogInfo) {
    if (e === void 0) { e = 'UNDEFINED ERROR'; }
    var errObj = new Error(e);
    if (_isReadyForFrontEndError()) {
        _handleError(e, additionalLogInfo, errObj);
    }
    else {
        // try again a little later, when window might be ready
        setTimeout(function () {
            _handleError(e, additionalLogInfo, errObj);
        }, WAIT_FOR_WIN_TIMEOUT_DURATION);
    }
};
exports.errorHandlerWithFrontendInform = errorHandlerWithFrontendInform;
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function _isReadyForFrontEndError() {
    var mainWin = (0, main_window_1.getWin)();
    var isAppReady = (0, main_window_1.getIsAppReady)();
    return mainWin && mainWin.webContents && isAppReady;
}
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function _handleError(e, additionalLogInfo, errObj) {
    var mainWin = (0, main_window_1.getWin)();
    var stack = errObj.stack;
    console.error('ERR', e);
    (0, main_1.log)(stack);
    (0, main_1.error)(e, stack);
    if (additionalLogInfo) {
        (0, main_1.log)('Additional Error info: ', additionalLogInfo);
    }
    if (_isReadyForFrontEndError()) {
        mainWin.webContents.send(ipc_events_const_1.IPC.ERROR, {
            error: e,
            errorStr: e && e.toString(),
            stack: stack,
        });
    }
    else {
        (0, main_1.error)('Electron Error: Frontend not loaded. Could not send error to renderer.');
        throw errObj;
    }
}
//# sourceMappingURL=error-handler-with-frontend-inform.js.map