"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.startApp = void 0;
var ipc_handler_1 = require("./ipc-handler");
var main_1 = require("electron-log/main");
var electron_1 = require("electron");
var path_1 = require("path");
var debug_1 = require("./debug");
var electronDl = require("electron-dl");
var ipc_events_const_1 = require("./shared-with-frontend/ipc-events.const");
var backup_1 = require("./backup");
var local_file_sync_1 = require("./local-file-sync");
var full_screen_blocker_1 = require("./full-screen-blocker");
var CONFIG_1 = require("./CONFIG");
var lazy_set_interval_1 = require("./shared-with-frontend/lazy-set-interval");
var indicator_1 = require("./indicator");
var various_shared_1 = require("./various-shared");
var main_window_1 = require("./main-window");
var ICONS_FOLDER = __dirname + '/assets/icons/';
var IS_MAC = process.platform === 'darwin';
// const DESKTOP_ENV = process.env.DESKTOP_SESSION;
// const IS_GNOME = DESKTOP_ENV === 'gnome' || DESKTOP_ENV === 'gnome-xorg';
var IS_DEV = process.env.NODE_ENV === 'DEV';
var isShowDevTools = IS_DEV;
var customUrl;
var isDisableTray = false;
var forceDarkTray = false;
var wasUserDataDirSet = false;
if (IS_DEV) {
    (0, main_1.log)('Starting in DEV Mode!!!');
}
var appIN = electron_1.app;
var mainWin;
var startApp = function () {
    // LOAD IPC STUFF
    (0, ipc_handler_1.initIpcInterfaces)();
    main_1.default.initialize();
    electron_1.app.commandLine.appendSwitch('enable-speech-dispatcher');
    // work around for #4375
    // https://github.com/johannesjo/super-productivity/issues/4375#issuecomment-2883838113
    // https://github.com/electron/electron/issues/46538#issuecomment-2808806722
    electron_1.app.commandLine.appendSwitch('gtk-version', '3');
    // NOTE: needs to be executed before everything else
    process.argv.forEach(function (val) {
        if (val && val.includes('--disable-tray')) {
            isDisableTray = true;
            (0, main_1.log)('Disable tray icon');
        }
        if (val && val.includes('--force-dark-tray')) {
            forceDarkTray = true;
            (0, main_1.log)('Force dark mode for tray icon');
        }
        if (val && val.includes('--user-data-dir=')) {
            var customUserDir = val.replace('--user-data-dir=', '').trim();
            (0, main_1.log)('Using custom directory for user data', customUserDir);
            electron_1.app.setPath('userData', customUserDir);
            wasUserDataDirSet = true;
        }
        if (val && val.includes('--custom-url=')) {
            customUrl = val.replace('--custom-url=', '').trim();
            (0, main_1.log)('Using custom url', customUrl);
        }
        if (val && val.includes('--dev-tools')) {
            isShowDevTools = true;
        }
    });
    // TODO remove at one point in the future and only leave the directory setting part
    // Special handling for snaps, since default user folder will cause problems when updating
    if (!wasUserDataDirSet &&
        process.platform === 'linux' &&
        process.env.SNAP &&
        process.env.SNAP_USER_COMMON) {
        // COPY LEGACY SNAP DATA TO COMMON DIRECTORY
        // -----------------------------------------
        var appName = electron_1.app.getName();
        var commonDir = process.env.SNAP_USER_COMMON;
        var newPath = (0, path_1.join)(commonDir, '.config', appName);
        // SET COMMON DIRECTORY AS USER DATA DIRECTORY
        // -------------------------------------------
        // set userDa dir to common data to avoid the data being accessed by the update process
        electron_1.app.setPath('userData', newPath);
        electron_1.app.setAppLogsPath();
    }
    (0, debug_1.initDebug)({ showDevTools: isShowDevTools }, IS_DEV);
    // NOTE: opening the folder crashes the mas build
    if (!IS_MAC) {
        electronDl({
            openFolderWhenDone: true,
            onCompleted: function (file) {
                if (mainWin) {
                    mainWin.webContents.send(ipc_events_const_1.IPC.ANY_FILE_DOWNLOADED, file);
                }
            },
        });
    }
    // Allow invalid certificates for jira requests
    appIN.on('certificate-error', function (event, webContents, url, err, certificate, callback) {
        (0, main_1.log)(err);
        event.preventDefault();
        callback(true);
    });
    // APP EVENT LISTENERS
    // -------------------
    appIN.on('ready', createMainWin);
    appIN.on('ready', function () { return (0, backup_1.initBackupAdapter)(); });
    appIN.on('ready', function () { return (0, local_file_sync_1.initLocalFileSyncAdapter)(); });
    appIN.on('ready', function () { return (0, full_screen_blocker_1.initFullScreenBlocker)(IS_DEV); });
    if (!isDisableTray) {
        appIN.on('ready', createIndicator);
    }
    appIN.on('activate', function () {
        // On OS X it's common to re-create a window in the app when the
        // dock icon is clicked and there are no other windows open.
        if (mainWin === null) {
            createMainWin();
        }
        else {
            showApp();
        }
    });
    appIN.on('ready', function () {
        var suspendStart;
        var sendIdleMsgIfOverMin = function (idleTime) {
            // sometimes when starting a second instance we get here although we don't want to
            if (!mainWin) {
                (0, main_1.info)('special case occurred when trackTimeFn is called even though, this is a second instance of the app');
                return;
            }
            // don't update if the user is about to close
            if (!appIN.isQuiting && idleTime > CONFIG_1.CONFIG.MIN_IDLE_TIME) {
                mainWin.webContents.send(ipc_events_const_1.IPC.IDLE_TIME, idleTime);
            }
        };
        var checkIdle = function () {
            return sendIdleMsgIfOverMin(electron_1.powerMonitor.getSystemIdleTime() * 1000);
        };
        // init time tracking interval
        (0, lazy_set_interval_1.lazySetInterval)(checkIdle, CONFIG_1.CONFIG.IDLE_PING_INTERVAL);
        electron_1.powerMonitor.on('suspend', function () {
            appIN.isLocked = true;
            suspendStart = Date.now();
            mainWin.webContents.send(ipc_events_const_1.IPC.SUSPEND);
        });
        electron_1.powerMonitor.on('lock-screen', function () {
            appIN.isLocked = true;
            suspendStart = Date.now();
            mainWin.webContents.send(ipc_events_const_1.IPC.SUSPEND);
        });
        electron_1.powerMonitor.on('resume', function () {
            appIN.isLocked = false;
            sendIdleMsgIfOverMin(Date.now() - suspendStart);
            mainWin.webContents.send(ipc_events_const_1.IPC.RESUME);
        });
        electron_1.powerMonitor.on('unlock-screen', function () {
            appIN.isLocked = false;
            sendIdleMsgIfOverMin(Date.now() - suspendStart);
            mainWin.webContents.send(ipc_events_const_1.IPC.RESUME);
        });
        electron_1.protocol.registerFileProtocol('file', function (request, callback) {
            var pathname = decodeURI(request.url.replace('file:///', ''));
            callback(pathname);
        });
    });
    appIN.on('will-quit', function () {
        // un-register all shortcuts.
        electron_1.globalShortcut.unregisterAll();
    });
    appIN.on('window-all-closed', function () {
        (0, main_1.log)('Quit after all windows being closed');
        // if (!IS_MAC) {
        electron_1.app.quit();
        // }
    });
    process.on('uncaughtException', function (err) {
        console.log(err);
        process.exit(333);
    });
    // AUTO-UPDATER
    // ------------
    // appIN.on('ready', () => {
    //  // init auto-updates
    //  log.info('INIT AUTO UPDATES');
    //  // log.info(autoUpdater.getFeedURL());
    //  autoUpdater.logger = log;
    //  autoUpdater.logger.transports.file.level = 'info';
    //  autoUpdater.checkForUpdatesAndNotify();
    // });
    //
    // autoUpdater.on('update-downloaded', (ev, info) => {
    //  log(ev);
    //  // Wait 5 seconds, then quit and install
    //  // In your application, you don't need to wait 5 seconds.
    //  // You could call autoUpdater.quitAndInstall(); immediately
    //  setTimeout(function() {
    //    autoUpdater.quitAndInstall();
    //  }, 5000)
    // });
    // HELPER FUNCTIONS
    // ----------------
    // eslint-disable-next-line prefer-arrow/prefer-arrow-functions
    function createIndicator() {
        (0, indicator_1.initIndicator)({
            app: electron_1.app,
            showApp: showApp,
            quitApp: various_shared_1.quitApp,
            ICONS_FOLDER: ICONS_FOLDER,
            forceDarkTray: forceDarkTray,
        });
    }
    // eslint-disable-next-line prefer-arrow/prefer-arrow-functions
    function createMainWin() {
        mainWin = (0, main_window_1.createWindow)({
            app: electron_1.app,
            IS_DEV: IS_DEV,
            ICONS_FOLDER: ICONS_FOLDER,
            quitApp: various_shared_1.quitApp,
            customUrl: customUrl,
        });
    }
    // eslint-disable-next-line prefer-arrow/prefer-arrow-functions
    function showApp() {
        (0, various_shared_1.showOrFocus)(mainWin);
    }
    // required for graceful closing
    // @see: https://github.com/electron/electron/issues/5708
    process.on('exit', function () {
        setTimeout(function () {
            (0, main_1.log)('Quit after process exit');
            electron_1.app.quit();
        }, 100);
    });
};
exports.startApp = startApp;
//# sourceMappingURL=start-app.js.map